@@ -38,11 +38,11 @@ asmlinkage void chacha20_avx512vl(u8 *out, const u8 *in, size_t len, const u32 k
 asmlinkage void poly1305_blocks_avx512(void *ctx, const u8 *inp, size_t len, u32 padbit);
 #endif
 
-static bool chacha20poly1305_use_ssse3 __read_mostly;
-static bool chacha20poly1305_use_avx __read_mostly;
-static bool chacha20poly1305_use_avx2 __read_mostly;
-static bool chacha20poly1305_use_avx512 __read_mostly;
-static bool chacha20poly1305_use_avx512vl __read_mostly;
+static bool chacha20poly1305_use_ssse3 __ro_after_init;
+static bool chacha20poly1305_use_avx __ro_after_init;
+static bool chacha20poly1305_use_avx2 __ro_after_init;
+static bool chacha20poly1305_use_avx512 __ro_after_init;
+static bool chacha20poly1305_use_avx512vl __ro_after_init;
 
 void __init chacha20poly1305_fpu_init(void)
 {
@@ -74,7 +74,7 @@ asmlinkage void poly1305_blocks_neon(void *ctx, const u8 *inp, size_t len, u32 p
 asmlinkage void poly1305_emit_neon(void *ctx, u8 mac[16], const u32 nonce[4]);
 asmlinkage void chacha20_neon(u8 *out, const u8 *in, size_t len, const u32 key[8], const u32 counter[4]);
 #endif
-static bool chacha20poly1305_use_neon __read_mostly;
+static bool chacha20poly1305_use_neon __ro_after_init;
 void __init chacha20poly1305_fpu_init(void)
 {
 #if defined(CONFIG_ARM64)
@@ -111,24 +111,19 @@ static inline u64 le64_to_cpuvp(const void *p)
 	return le64_to_cpup(p);
 }
 
-static inline u32 rotl32(u32 v, u8 n)
-{
-	return (v << n) | (v >> (sizeof(v) * 8 - n));
-}
-
 struct chacha20_ctx {
 	u32 state[CHACHA20_BLOCK_SIZE / sizeof(u32)];
 } __aligned(32);
 
 #define QUARTER_ROUND(x, a, b, c, d) ( \
 	x[a] += x[b], \
-	x[d] = rotl32((x[d] ^ x[a]), 16), \
+	x[d] = rol32((x[d] ^ x[a]), 16), \
 	x[c] += x[d], \
-	x[b] = rotl32((x[b] ^ x[c]), 12), \
+	x[b] = rol32((x[b] ^ x[c]), 12), \
 	x[a] += x[b], \
-	x[d] = rotl32((x[d] ^ x[a]), 8), \
+	x[d] = rol32((x[d] ^ x[a]), 8), \
 	x[c] += x[d], \
-	x[b] = rotl32((x[b] ^ x[c]), 7) \
+	x[b] = rol32((x[b] ^ x[c]), 7) \
 )
 
 #define C(i, j) (i * 4 + j)
@@ -159,28 +154,29 @@ struct chacha20_ctx {
 	DOUBLE_ROUND(x) \
 )
 
-static void chacha20_block_generic(struct chacha20_ctx *ctx, void *stream)
+#define EXPAND_32_BYTE_K 0x61707865, 0x3320646e, 0x79622d32, 0x6b206574
+
+static void chacha20_block_generic(struct chacha20_ctx *ctx, __le32 *stream)
 {
 	u32 x[CHACHA20_BLOCK_SIZE / sizeof(u32)];
-	__le32 *out = stream;
 	int i;
 
-	for (i = 0; i < ARRAY_SIZE(x); i++)
+	for (i = 0; i < ARRAY_SIZE(x); ++i)
 		x[i] = ctx->state[i];
 
 	TWENTY_ROUNDS(x);
 
-	for (i = 0; i < ARRAY_SIZE(x); i++)
-		out[i] = cpu_to_le32(x[i] + ctx->state[i]);
+	for (i = 0; i < ARRAY_SIZE(x); ++i)
+		stream[i] = cpu_to_le32(x[i] + ctx->state[i]);
 
-	ctx->state[12]++;
+	++ctx->state[12];
 }
 
 static void hchacha20_generic(u8 derived_key[CHACHA20POLY1305_KEYLEN], const u8 nonce[16], const u8 key[CHACHA20POLY1305_KEYLEN])
 {
 	__le32 *out = (__force __le32 *)derived_key;
 	u32 x[] = {
-		0x61707865, 0x3320646e, 0x79622d32, 0x6b206574,
+		EXPAND_32_BYTE_K,
 		le32_to_cpuvp(key + 0), le32_to_cpuvp(key + 4), le32_to_cpuvp(key + 8), le32_to_cpuvp(key + 12),
 		le32_to_cpuvp(key + 16), le32_to_cpuvp(key + 20), le32_to_cpuvp(key + 24), le32_to_cpuvp(key + 28),
 		le32_to_cpuvp(nonce +  0), le32_to_cpuvp(nonce +  4), le32_to_cpuvp(nonce +  8), le32_to_cpuvp(nonce + 12)
@@ -211,7 +207,7 @@ static inline void hchacha20(u8 derived_key[CHACHA20POLY1305_KEYLEN], const u8 n
 }
 
 #define chacha20_initial_state(key, nonce) {{ \
-	0x61707865, 0x3320646e, 0x79622d32, 0x6b206574, \
+	EXPAND_32_BYTE_K, \
 	le32_to_cpuvp((key) + 0), le32_to_cpuvp((key) + 4), le32_to_cpuvp((key) + 8), le32_to_cpuvp((key) + 12), \
 	le32_to_cpuvp((key) + 16), le32_to_cpuvp((key) + 20), le32_to_cpuvp((key) + 24), le32_to_cpuvp((key) + 28), \
 	0, 0, le32_to_cpuvp((nonce) +  0), le32_to_cpuvp((nonce) + 4) \
@@ -219,7 +215,7 @@ static inline void hchacha20(u8 derived_key[CHACHA20POLY1305_KEYLEN], const u8 n
 
 static void chacha20_crypt(struct chacha20_ctx *ctx, u8 *dst, const u8 *src, u32 bytes, bool have_simd)
 {
-	u8 buf[CHACHA20_BLOCK_SIZE];
+	__le32 buf[CHACHA20_BLOCK_SIZE / sizeof(__le32)];
 
 	if (!have_simd
 #if defined(CONFIG_X86_64)
@@ -274,13 +270,13 @@ no_simd:
 
 	while (bytes >= CHACHA20_BLOCK_SIZE) {
 		chacha20_block_generic(ctx, buf);
-		crypto_xor(dst, buf, CHACHA20_BLOCK_SIZE);
+		crypto_xor(dst, (u8 *)buf, CHACHA20_BLOCK_SIZE);
 		bytes -= CHACHA20_BLOCK_SIZE;
 		dst += CHACHA20_BLOCK_SIZE;
 	}
 	if (bytes) {
 		chacha20_block_generic(ctx, buf);
-		crypto_xor(dst, buf, bytes);
+		crypto_xor(dst, (u8 *)buf, bytes);
 	}
 }
 
@@ -855,3 +851,4 @@ bool xchacha20poly1305_decrypt(u8 *dst, const u8 *src, const size_t src_len,
 }
 
 #include "../selftest/chacha20poly1305.h"
+#include "../selftest/poly1305.h"
